@@ -16,9 +16,9 @@
  * Authored by: Pete Woods <pete.woods@canonical.com>
  */
 
-#include <unity/gmenuharness/MatchResult.h>
-#include <unity/gmenuharness/MatchUtils.h>
-#include <unity/gmenuharness/MenuItemMatcher.h>
+#include <lomiri/gmenuharness/MatchResult.h>
+#include <lomiri/gmenuharness/MatchUtils.h>
+#include <lomiri/gmenuharness/MenuItemMatcher.h>
 
 #include <iostream>
 #include <vector>
@@ -26,7 +26,7 @@
 
 using namespace std;
 
-namespace unity
+namespace lomiri
 {
 
 namespace gmenuharness
@@ -163,8 +163,7 @@ struct MenuItemMatcher::Priv
         }
 
         // match the last N items
-        size_t j;
-        for (size_t i = count - m_items.size(), j = 0; i < count && j < m_items.size(); ++i, ++j)
+        for (size_t i = count - m_items.size(), j = 0; i < (unsigned int) count && j < m_items.size(); ++i, ++j)
         {
             const auto& matcher = m_items.at(j);
             matcher.match(matchResult, location, menu, actions, i);
@@ -197,6 +196,8 @@ struct MenuItemMatcher::Priv
 
     shared_ptr<bool> m_isToggled;
 
+    shared_ptr<bool> m_isEnabled;
+
     vector<MenuItemMatcher> m_items;
 
     vector<pair<string, shared_ptr<GVariant>>> m_activations;
@@ -254,6 +255,7 @@ MenuItemMatcher& MenuItemMatcher::operator=(const MenuItemMatcher& other)
     p->m_not_exist_attributes = other.p->m_not_exist_attributes;
     p->m_pass_through_attributes = other.p->m_pass_through_attributes;
     p->m_isToggled = other.p->m_isToggled;
+    p->m_isEnabled = other.p->m_isEnabled;
     p->m_linkType = other.p->m_linkType;
     p->m_items = other.p->m_items;
     p->m_activations = other.p->m_activations;
@@ -403,6 +405,12 @@ MenuItemMatcher& MenuItemMatcher::toggled(bool isToggled)
     return *this;
 }
 
+MenuItemMatcher& MenuItemMatcher::enabled(bool isEnabled)
+{
+    p->m_isEnabled = make_shared<bool>(isEnabled);
+    return *this;
+}
+
 MenuItemMatcher& MenuItemMatcher::submenu()
 {
     p->m_linkType = LinkType::submenu;
@@ -485,6 +493,7 @@ void MenuItemMatcher::match(
     bool isCheckbox = false;
     bool isRadio = false;
     bool isToggled = false;
+    bool isEnabled = true;
 
     pair<string, string> idPair;
     shared_ptr<GActionGroup> actionGroup;
@@ -510,6 +519,9 @@ void MenuItemMatcher::match(
             isToggled = g_variant_get_boolean(state.get());
             isCheckbox = true;
         }
+
+        isEnabled = g_action_group_get_action_enabled(actionGroup.get(),
+                                                      idPair.second.c_str()) == TRUE;
     }
 
     Type actualType = Type::plain;
@@ -558,7 +570,7 @@ void MenuItemMatcher::match(
                 ++nb_icons;
             }
 
-            if (nb_icons != iter->second.size())
+            if ((unsigned int) nb_icons != iter->second.size())
             {
                 matchResult.failure(
                            location,
@@ -809,6 +821,14 @@ void MenuItemMatcher::match(
                         + ", but found " + bool_to_string(isToggled));
     }
 
+    if (p->m_isEnabled && (*p->m_isEnabled) != isEnabled)
+    {
+        matchResult.failure(
+                location,
+                "Expected enabled = " + bool_to_string(*p->m_isEnabled)
+                        + ", but found " + bool_to_string(isEnabled));
+    }
+
     if (!matchResult.success())
     {
         return;
@@ -996,7 +1016,8 @@ void MenuItemMatcher::match(
             }
             else
             {
-                g_action_group_activate_action(tmpActionGroup.get(), tmpIdPair.second.c_str(), nullptr);
+                auto actionTarget = get_attribute(menuItem, G_MENU_ATTRIBUTE_TARGET);
+                g_action_group_activate_action(tmpActionGroup.get(), tmpIdPair.second.c_str(), actionTarget.get());
             }
 
             // FIXME this is a dodgy way to ensure the activation gets dispatched
@@ -1007,4 +1028,4 @@ void MenuItemMatcher::match(
 
 }   // namepsace gmenuharness
 
-}   // namespace unity
+}   // namespace lomiri
